#############################################################################
##
#W algebra.gi                                               Laurent Bartholdi
##
#H   @(#)$Id: algebra.gi,v 1.16 2008/12/17 10:46:07 gap Exp $
##
#Y Copyright (C) 2007, Laurent Bartholdi
##
#############################################################################
##
##  This file implements self-similar associative algebras.
##
#############################################################################

InstallAccessToGenerators(IsFRAlgebra,
        "(FR) for a FR algebra",GeneratorsOfAlgebra);

InstallAccessToGenerators(IsFRAlgebraWithOne,
        "(FR) for a FR algebra-with-one",GeneratorsOfAlgebraWithOne);

#############################################################################
##
#M AlphabetOfFRAlgebra
##
InstallMethod(AlphabetOfFRAlgebra, "(FR) for an FR algebra",
        [IsFRAlgebra],
        G->AlphabetOfFRObject(Representative(G)));
#############################################################################

#############################################################################
##
#O SCAlgebra
#O SCAlgebraWithOne
##
InstallMethod(AlgebraHomomorphismByFunction, "(FR) for two algebras and a function",
        [IsAlgebra,IsAlgebra,IsFunction],
        function(S,R,f)
    return Objectify(TypeOfDefaultGeneralMapping(S,R,
                   IsSPGeneralMapping and IsAlgebraGeneralMapping
                   and IsMappingByFunctionRep), rec(fun:=f));
end);

InstallMethod(AlgebraWithOneHomomorphismByFunction, "(FR) for two algebras and a function",
        [IsAlgebraWithOne,IsAlgebraWithOne,IsFunction],
        function(S,R,f)
    return Objectify(TypeOfDefaultGeneralMapping(S,R,
                   IsSPGeneralMapping and IsAlgebraWithOneGeneralMapping
                   and IsMappingByFunctionRep), rec(fun:=f));
end);
#############################################################################

#############################################################################
##
#O SCAlgebra
#O SCAlgebraWithOne
##
InstallMethod(SCAlgebraNC, "(FR) for a linear machine",
        [IsLinearFRMachine],
        function(M)
    local a, g;
    a := Objectify(NewType(CollectionsFamily(FREFamily(M)),
                 IsFRAlgebra and IsAttributeStoringRep),
                 rec());
    SetLeftActingDomain(a,LeftActingDomain(M));
    g := List(GeneratorsOfFRMachine(M),x->FRElement(M,x));
    SetGeneratorsOfLeftOperatorRing(a,g);
    SetCorrespondence(a,g);
    SetUnderlyingFRMachine(a,M);
    if IsVectorFRMachineRep(M) then
        SetFilterObj(a,IsVectorFRElementSpace);
    fi;
    SetFilterObj(a,IsLinearFRElementSpace);
    return a;
end);

InstallMethod(SCAlgebra, "(FR) for a linear machine",
        [IsLinearFRMachine], SCAlgebraNC);

InstallMethod(SCAlgebraWithOneNC, "(FR) for a linear machine",
        [IsLinearFRMachine],
        function(M)
    local a, g;
    a := Objectify(NewType(CollectionsFamily(FREFamily(M)),
                 IsFRAlgebraWithOne and IsAttributeStoringRep),
                 rec());
    SetLeftActingDomain(a,LeftActingDomain(M));
    g := List(GeneratorsOfFRMachine(M),x->FRElement(M,x));
    SetGeneratorsOfLeftOperatorRingWithOne(a,g);
    SetCorrespondence(a,g);
    SetUnderlyingFRMachine(a,M);
    if IsVectorFRMachineRep(M) then
        SetFilterObj(a,IsVectorFRElementSpace);
    fi;
    SetFilterObj(a,IsLinearFRElementSpace);
    return a;
end);

InstallMethod(SCAlgebraWithOne, "(FR) for a linear machine",
        [IsLinearFRMachine],
        function(M)
    local a, g, p;
    a := Objectify(NewType(CollectionsFamily(FREFamily(M)),
                 IsFRAlgebraWithOne and IsAttributeStoringRep),
                 rec());
    SetLeftActingDomain(a,LeftActingDomain(M));
    g := List(GeneratorsOfFRMachine(M),x->FRElement(M,x));
    SetCorrespondence(a,g);
    g := DuplicateFreeList(g);
    if Length(g)>=1 then
        p := Position(g,One(g[1]));
        if p<>fail then Remove(g,p); fi;
    fi;
    SetGeneratorsOfLeftOperatorRingWithOne(a,g);
    SetUnderlyingFRMachine(a,M);
    if IsVectorFRMachineRep(M) then
        SetFilterObj(a,IsVectorFRElementSpace);
    fi;
    SetFilterObj(a,IsLinearFRElementSpace);
    return a;
end);
############################################################################

#############################################################################
##
#F FRAlgebra
#F FRAlgebraWithOne
##
FR_LOCAL.TOTALDEGREE := function(x)
    local d, i;
    d := -1;
    x := ExtRepOfObj(x)[2];
    for i in x{[1,3..Length(x)-1]} do
        d := Maximum(d,Sum(i{[2,4..Length(i)]}));
    od;
    return d;
end;

FR_LOCAL.STRINGSTOLMACHINE := function(r,arg,creator)
    local temp, i, j, gens, transitions, output;
    
    if not IsRing(r) or not ForAll(arg,IsString) then
        Error("<arg> should contain a ring and strings\n");
    fi;
    temp := List(arg, x->SplitString(x,"="));
    if ForAny(temp,x->Size(x)<>2) then
        Error("<arg> should have the form a=[[...]...]\n");
    fi;
    gens := List(temp, x->x[1]);
    if Size(Set(gens)) <> Size(gens) then
        Error("all generators should have a distinct name\n");
    fi;
    FR_LOCAL.STRINGSTOMACHINE_HOLDER := FreeAssociativeAlgebraWithOne(r,gens);
    
    transitions := [];
    output := [];
    for temp in List(temp,x->x[2]) do
        temp := SplitString(temp,":");
        if Length(temp)=2 then
            Add(output,FR_LOCAL.STRINGATOM2GAP(temp[2])*One(r));
        else
            Add(output,One(r));
        fi;
        temp := FR_LOCAL.STRINGWORD2GAP(gens,"GeneratorsOfAlgebraWithOne",temp[1])*One(FR_LOCAL.STRINGSTOMACHINE_HOLDER);
        if not IsMatrix(temp) then
            Error("<arg> should have the form a=[[...]...]\n");
        fi;
        Add(transitions,temp);
    od;

    i := AlgebraMachine(r,FR_LOCAL.STRINGSTOMACHINE_HOLDER,transitions,output);
    if ValueOption("IsVectorElement")=true or
       (ForAll(Flat(transitions),x->FR_LOCAL.TOTALDEGREE(x)<=1) and ValueOption("IsAlgebraElement")<>true) then
        i := AsVectorMachine(i);
        i := List(Correspondence(i),x->FRElement(i,x));
        for temp in [1..Length(gens)] do
            SetName(i[temp],gens[temp]);
        od;
        i := creator(r,i);
    else
        i := creator(r,List(GeneratorsOfFRMachine(i),x->FRElement(i,x)));
    fi;
    SetIsStateClosedFRAlgebra(i,true);
    return i;
end;

InstallGlobalFunction(FRAlgebra,
        function(arg)
    return FR_LOCAL.STRINGSTOLMACHINE(arg[1],arg{[2..Length(arg)]},Algebra);
end);

InstallGlobalFunction(FRAlgebraWithOne,
        function(arg)
    return FR_LOCAL.STRINGSTOLMACHINE(arg[1],arg{[2..Length(arg)]},AlgebraWithOne);
end);

InstallMethod(AssignGeneratorVariables, "(FR) for an FR algebra",
        [IsFRAlgebra],
        function(G)
    FR_LOCAL.ASSIGNGENERATORVARIABLES(GeneratorsOfAlgebra(G));
end);

InstallMethod(AssignGeneratorVariables, "(FR) for an FR algebra with one",
        [IsFRAlgebraWithOne],
        function(G)
    FR_LOCAL.ASSIGNGENERATORVARIABLES(GeneratorsOfAlgebraWithOne(G));
end);
############################################################################

#############################################################################
##
#O ThinnedAlgebra
#O ThinnedAlgebraWithOne
##
InstallMethod(ThinnedAlgebra, "(FR) for a ring and a FR semigroup",
        [IsRing, IsFRSemigroup],
        function(r,G)
    local a, g;
    g := List(GeneratorsOfSemigroup(G),x->AsLinearElement(r,x));
    a := Objectify(NewType(CollectionsFamily(FamilyObj(g[1])),
                 IsFRAlgebra and IsAttributeStoringRep),
                 rec());
    SetLeftActingDomain(a,LeftActingDomain(g[1]));
    SetGeneratorsOfLeftOperatorRing(a,g);
    if HasSize(G) and Size(G)=infinity then
        SetDimension(G,infinity);
    fi;
    return a;
end);

InstallMethod(ThinnedAlgebraWithOne, "(FR) for a ring and a FR monoid",
        [IsRing, IsFRMonoid],
        function(r,G)
    local a, g;
    g := List(GeneratorsOfMonoid(G),x->AsLinearElement(r,x));
    a := Objectify(NewType(CollectionsFamily(FamilyObj(g[1])),
                 IsFRAlgebraWithOne and IsAttributeStoringRep),
                 rec());
    SetLeftActingDomain(a,LeftActingDomain(g[1]));
    SetGeneratorsOfLeftOperatorRingWithOne(a,g);
    SetAugmentationIdeal(a,TwoSidedIdealByGenerators(a,List(g,x->x-One(a))));
    return a;
end);

InstallMethod(Embedding, "(FR) for a semigroup and a FR algebra",
        [IsFRSemigroup, IsFRAlgebra],
        function(G,A)
    if IsGroup(G) then
        return GroupHomomorphismByFunction(G,A,x->AsLinearElement(LeftActingDomain(A),x));
    else
        return MagmaHomomorphismByFunctionNC(G,A,x->AsLinearElement(LeftActingDomain(A),x));
    fi;
end);
############################################################################

#############################################################################
##
#O Nillity
##
FR_LOCAL.ISNIL_GENERIC := function(x) # returns false or 2-powers of x till 0
    local powx, rank, oldrank, ring;
    
    powx := [x];
    if IsMatrix(x) then
        ring := DefaultRing(x[1][1]);
    else
        ring := DefaultRing(x);
    fi;
    if IsMatrix(x) and (HasIsIntegralRing(x) and IsIntegralRing(x)) then
        rank := RankMat(x);
        while not IsZero(x) do
            x := x*x;
            Add(powx,x);
            oldrank := rank;
            rank := RankMat(x);
            if rank=oldrank then return false; fi;
        od;
        # certainly not a nil element if has non-zero generalized eigenspace
    else
        if HasIsIntegralRing(x) and IsIntegralRing(x) then
            if IsZero(x) then return [x]; else return false; fi;
        fi;
        while not IsZero(x) do
            if IsOne(x) then return false; fi;
            x := x*x;
            Add(powx,x);
        od;
    fi;
    return powx;
end;

FR_LOCAL.ISNIL_FR := function(x)
    local powx, deg, seen, recur;
    
    deg := Dimension(AlphabetOfFRObject(x));
    powx := [x];
    seen := NewDictionary(x,false);
    
    recur := function(x,level)
        local c, d, depth, olddepth;
        olddepth := 0;
        if KnowsDictionary(seen,x) then
            return false;
        fi;
        while not IsZero(x) do
            # first see if element is triangular
        
            d := DecompositionOfFRElement(x);
            c := List(Filtered(List(EquivalenceClasses(StronglyConnectedComponents(TransitiveClosureBinaryRelation(BinaryRelationOnPoints(List([1..deg],i->Filtered([1..deg],j->not IsZero(d[i][j]))))))),AsList),x->Length(x)=1),c->d[c[1]][c[1]]);
            if ForAny(c,x->KnowsDictionary(seen,x)) then
                return false;   # some entry of x^(2^n) is a smaller power of x
            fi;
            if Length(c)=deg then
                return ForAll(c,x->recur(x,level+1));
            fi;
            
            # now see if a projection is non-nil
        
            if IsVectorFRMachineRep(x) then
                depth := LogInt(Dimension(StateSet(x))+1,deg)+1;
            else
                depth := LogInt(Length(Flat(ExtRepOfObj(InitialState(x)))),deg)+2;
            fi;
            if depth > olddepth then
                olddepth := depth;
                if FR_LOCAL.ISNIL_GENERIC(Activity(x,depth))=false then
                    return false;
                fi;
            fi;
            AddDictionary(seen,x);
            x := x*x;
            if level=0 then Add(powx,x); fi;
        od;
        return true;
    end;

    if recur(x,0) then
        return powx;
    else
        return false;
    fi;
end;

FR_LOCAL.NILLITY := function(x,isnil)
    local powx, pown, n, y;
    
    powx := isnil(x);
    
    if powx=false then return infinity; fi;
    
    Append(powx,FR_LOCAL.ISNIL_GENERIC(Remove(powx))); # get all 2-powers of x
    Remove(powx);
    pown := List([0..Length(powx)-1],i->2^i);
    
    if IsEmpty(powx) then return 1; fi;
    
    x := Remove(powx);
    n := Remove(pown)+1;
    while not IsEmpty(powx) do
        y := x*Remove(powx);
        if IsZero(y) then
            Remove(pown);
        else
            x := y;
            n := n+Remove(pown);
        fi;
    od;
    return n;
end;

InstallMethod(Nillity, "(FR) for an associative element",
        [IsAssociativeElement],
        x->FR_LOCAL.NILLITY(x,FR_LOCAL.ISNIL_GENERIC));

InstallMethod(Nillity, "(FR) for an FR element",
        [IsLinearFRElement],
        x->FR_LOCAL.NILLITY(x,FR_LOCAL.ISNIL_FR));

InstallMethod(IsNilpotentElement, "(FR) for an associative element",
        [IsAssociativeElement],
        x->FR_LOCAL.ISNIL_GENERIC(x)<>false);

InstallMethod(IsNilpotentElement, "(FR) for an FR element",
        [IsLinearFRElement],
        x->FR_LOCAL.ISNIL_FR(x)<>false);
#############################################################################

#############################################################################
##
#O MatrixQuotient
##
InstallMethod(MatrixQuotient, "(FR) for a FR algebra and a level",
        [IsFRAlgebra,IsInt],
        function(A,n)
    return Algebra(LeftActingDomain(A),List(GeneratorsOfAlgebra(A),x->Activity(x,n)));
end);

InstallMethod(EpimorphismMatrixQuotient, "(FR) for a FR algebra and a level",
        [IsFRAlgebra,IsInt],
        function(A,n)
    local Q;
    Q := MatrixQuotient(A,n);
    return AlgebraHomomorphismByFunction(A,Q,x->Activity(x,n));
end);

InstallMethod(MatrixQuotient, "(FR) for a FR algebra-with-one and a level",
        [IsFRAlgebraWithOne,IsInt],
        function(A,n)
    local Q;
    Q := AlgebraWithOne(LeftActingDomain(A),List(GeneratorsOfAlgebraWithOne(A),x->Activity(x,n)));
    if HasAugmentationIdeal(A) then
        SetAugmentationIdeal(Q,Ideal(Q,List(GeneratorsOfIdeal(AugmentationIdeal(A)),x->Activity(x,n))));
    fi;
    return Q;
end);

InstallMethod(EpimorphismMatrixQuotient, "(FR) for a FR algebra-with-one and a level",
        [IsFRAlgebraWithOne,IsInt],
        function(A,n)
    local Q;
    Q := MatrixQuotient(A,n);
    return AlgebraWithOneHomomorphismByFunction(A,Q,x->Activity(x,n));
end);
############################################################################

#############################################################################
##
#M View
##
InstallMethod(ViewObj, "(FR) for an FR algebra",
        [IsFRAlgebra],
        function(A)
    local n;
    n := Length(GeneratorsOfAlgebra(A));
    Print("<self-similar algebra on alphabet ", LeftActingDomain(A), "^",
          Dimension(AlphabetOfFRAlgebra(A)), " with ",n," generator");
    if n>1 then Print("s"); fi;
    if HasDimension(A) then Print(", of dimension ",Dimension(A)); fi;
    Print(">");
end);

InstallMethod(ViewObj, "(FR) for an FR algebra with one",
        [IsFRAlgebraWithOne],
        function(A)
    local n;
    n := Length(GeneratorsOfAlgebraWithOne(A));
    Print("<self-similar algebra-with-one on alphabet ",
          LeftActingDomain(A), "^", Dimension(AlphabetOfFRAlgebra(A)),
          " with ",n," generator");
    if n>1 then Print("s"); fi;
    if HasDimension(A) then Print(", of dimension ",Dimension(A)); fi;
    Print(">");
end);
############################################################################

#E algebra.gi . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
