##############################################################################
##
#W  gpd.tst                       Gpd Package                    Chris Wensley
##
#H  @(#)$Id: gpd.tst,v 1.05 2008/11/21 gap Exp $
##
#Y  Copyright (C) 1999,2001 University of St. Andrews, North Haugh,
#Y                          St. Andrews, Fife KY16 9SS, Scotland
##

gap> START_TEST( "Testing examples in Chapter 4 of the Gpd manual" );

gap> gpd_infolevel_saved := InfoLevel( InfoGpd );; 
gap> SetInfoLevel( InfoGpd, 0 );; 

gap> d8 := Group( (1,2,3,4), (1,3) );;
gap> SetName( d8, "d8" );
gap> Gd8 := Groupoid( d8, [-9,-8,-7] );
single piece groupoid: < d8, [ -9, -8, -7 ] >
gap> c6 := Group( (5,6,7)(8,9) );;
gap> SetName( c6, "c6" );
gap> Gc6 := DomainWithSingleObject( c6, -6 );
single piece groupoid: < c6, [ -6 ] >
gap> Gd8c6 := UnionOfPieces( [ Gd8, Gc6 ] );;
gap> Display( Gd8c6 );
groupoid with 2 pieces:
< objects: [ -9, -8, -7 ]
    group: d8 = <[ (1,2,3,4), (1,3) ]> >
< objects: [ -6 ]
    group: c6 = <[ (5,6,7)(8,9) ]> >
gap> SetName( Gd8, "Gd8" );
gap> SetName( Gc6, "Gc6" );
gap> SetName( Gd8c6, "Gd8+Gc6" );

## SubSection 4.1.2
gap> Pieces( Gd8c6 );
[ Gd8, Gc6 ]
gap> ObjectList( Gd8c6 );
[ -9, -8, -7, -6 ]

## SubSection 4.1.4
gap> f2 := FreeGroup( 2 );;
gap> SetName( f2, "f2" );;
gap> Gf2 := Groupoid( f2, -22 );;
gap> q8 := SmallGroup( 8, 4 );;
gap> SetName( q8, "q8" );;
gap> Gq8 := Groupoid( q8, [ -28, -27 ] );;
gap> SetName( Gq8, "Gq8" );;
gap> Gq8f2 := Groupoid( [ Gf2, Gq8 ] );;
gap> G4 := UnionOfPieces( [ Gd8c6, Gf2, Gq8 ] );
groupoid with pieces:
1:  Gq8
2:  single piece groupoid: < f2, [ -22 ] >
3:  Gd8
4:  Gc6

gap> Display( G4 );
groupoid with 4 pieces:
< objects: [ -28, -27 ]
    group: q8 = <[ f1, f2, f3 ]> >
< objects: [ -22 ]
    group: f2 = <[ f1, f2 ]> >
< objects: [ -9, -8, -7 ]
    group: d8 = <[ (1,2,3,4), (1,3) ]> >
< objects: [ -6 ]
    group: c6 = <[ (5,6,7)(8,9) ]> >
gap> G5 := Groupoid( [ Gq8, Gf2, Gd8c6 ] );;
gap> G4 = G5;
true

## SubSection 4.1.5
gap> e1 := GroupoidElement( Gd8, (1,2)(3,4), -9, -8 );
[(1,2)(3,4) : -9 -> -8]
gap> e2 := GroupoidElement( Gd8, (1,3), -8, -7 );;
gap> Print( [ Arrowelt(e2), Arrowtail(e2), Arrowhead(e2) ], "\n" );
[ (1,3), -8, -7 ]
gap> prod := e1*e2;
[(1,2,3,4) : -9 -> -7]
gap> e3 := GroupoidElement( Gd8, (1,3)(2,4), -7, -9 );;
gap> cycle := prod*e3;
[(1,4,3,2) : -9 -> -9]
gap> cycle^2;
[(1,3)(2,4) : -9 -> -9]
gap> Order(cycle);
4
gap> ConjugateGroupoidElement( cycle, e1 );
[(1,2,3,4) : -8 -> -8]
gap> [ Size( Gd8 ), Size( Gc6 ), Size( Gd8c6 ), Size( Gq8f2 ) ];
[ 72, 6, 78, infinity ]


### Section 4.2 : Subgroupoids ###

## SubSection 4.2.1
gap> c4d := Subgroup( d8, [ (1,2,3,4) ] );;
gap> k4d := Subgroup( d8, [ (1,2)(3,4), (1,3)(2,4) ] );;
gap> SetName( c4d, "c4d" );  SetName( k4d, "k4d" );
gap> Ud8 := Subgroupoid( Gd8, [ [ k4d, [-9] ], [ c4d, [-8,-7] ] ] );;
gap> SetName( Ud8, "Ud8" );
gap> Display( Ud8 );
groupoid with 2 pieces:
< objects: [ -9 ]
    group: k4d = <[ (1,2)(3,4), (1,3)(2,4) ]> >
< objects: [ -8, -7 ]
    group: c4d = <[ (1,2,3,4) ]> >


### Section 2.3 : Operations on Groupoid Elements ###

## SubSection 2.3.1
gap> star9 := ObjectStar( Gd8, -9 );
<star at [ -9 ] with group d8>
gap> for e in star9 do
gap>     if ( Order( e![1] ) = 4 ) then Print( e, "\n" ); fi;
gap> od;
[(1,4,3,2) : -9 -> -9]
[(1,4,3,2) : -9 -> -8]
[(1,4,3,2) : -9 -> -7]
[(1,2,3,4) : -9 -> -9]
[(1,2,3,4) : -9 -> -8]
[(1,2,3,4) : -9 -> -7]
gap> costar6 := ObjectCostar( Gc6, -6 );
<costar at [ -6 ] with group c6>
gap> for e in costar6 do
gap>     if ( Order( e![1] ) = 6 ) then Print( e, "\n" ); fi;
gap> od;
[(5,6,7)(8,9) : -6 -> -6]
[(5,7,6)(8,9) : -6 -> -6]
gap> hset78 := Homset( Ud8, -7, -8 );
<homset -7 -> -8 with group c4d>
gap> for e in hset78 do  Print( e, "\n" );  od;
[() : -7 -> -8]
[(1,3)(2,4) : -7 -> -8]
[(1,4,3,2) : -7 -> -8]
[(1,2,3,4) : -7 -> -8]

## SubSection 2.3.2
gap> i7 := IdentityElement( Gd8, -7 );;
gap> i8 := IdentityElement( Gd8, -8 );;
gap> L := [ i7, i8 ];;
gap> for e in hset78 do  Add( L, i7*e*i8 = e );;  od;;
gap> L;
[ [() : -7 -> -7], [() : -8 -> -8], true, true, true, true ]

### Section 2.4 : Left, Right and Double Cosets ###

## SubSection 2.4.1
gap> re2 := RightCoset( Gd8, Ud8, e2 );
RightCoset(single piece groupoid: < c4d, [ -8, -7 ] >,[(1,3) : -8 -> -7])
gap> for x in re2 do Print( x, "\n" ); od;
[(1,3) : -8 -> -7]
[(1,3) : -7 -> -7]
[(2,4) : -8 -> -7]
[(2,4) : -7 -> -7]
[(1,4)(2,3) : -8 -> -7]
[(1,4)(2,3) : -7 -> -7]
[(1,2)(3,4) : -8 -> -7]
[(1,2)(3,4) : -7 -> -7]
gap> rcrd8 := RightCosetRepresentatives( Gd8, Ud8 );
[ [() : -9 -> -9], [() : -9 -> -8], [() : -9 -> -7], [(2,4) : -9 -> -9],
  [(2,4) : -9 -> -8], [(2,4) : -9 -> -7], [() : -8 -> -9], [() : -8 -> -8],
  [() : -8 -> -7], [(2,4) : -8 -> -9], [(2,4) : -8 -> -8], [(2,4) : -8 -> -7]
 ]
gap> lcr7 := LeftCosetRepresentativesFromObject( Gd8, Ud8, -7 );
[ [() : -7 -> -9], [(2,4) : -7 -> -9], [() : -7 -> -8], [(2,4) : -7 -> -8] ]


gap> SetInfoLevel( InfoGpd, gpd_infolevel_saved );;  

gap> STOP_TEST( "gpd.tst", 10000 );

#############################################################################
##
#E  gpd.tst . . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
