#############################################################################
##
##  HAPPRIME - happrime.tst
##  Test the package with some examples
##  Paul Smith
##
##  Copyright (C)  2008-2009
##  Paul Smith
##  National University of Ireland Galway
##
##  This file is part of HAPprime. 
## 
##  HAPprime is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
## 
##  HAPprime is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
## 
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
## 
##  $Id: happrime.tst 356 2008-12-10 12:40:51Z pas $
##
#############################################################################

gap> START_TEST("HAPprime version 0.3.2 general tests");
gap> 
gap> LoadPackage("HAP");;
gap> LoadPackage("HAPPrime");;
gap> 
gap> # Set the info level to zero to suppress some of the output
gap> infolevel := InfoLevel(InfoHAPprime);;
gap> SetInfoLevel(InfoHAPprime, 0);
gap> ##
gap> ## Test ResolutionPrimePowerGroupGF by calculating the module ranks
gap> ## in a minimal resolution
gap> R := ResolutionPrimePowerGroupGF( SmallGroup(32, 8), 5 );;
gap> Print(ResolutionModuleRanks(R), "\n"); 
[ 2, 3, 4, 4, 5 ]
gap> ##
gap> ## Test ResolutionPrimePowerGroupRadical by seeing that it gives the 
gap> ## correct PoincareSeriesAutoMem
gap> R := ResolutionPrimePowerGroupRadical( SmallGroup(128, 154), 8 );;
gap> Print(PoincareSeries(R, 8), "\n");
(1)/(x_1^4-2*x_1^3+2*x_1^2-2*x_1+1)
gap> ##
gap> ## Test ResolutionPrimePowerGroupAutoMem by seeing comparing the torsion
gap> ## coefficients of the homology
gap> ## Note that this is unlikely to switch to the GF version, but
gap> ## it at least checks that the function exists
gap> R := ResolutionPrimePowerGroupAutoMem( SmallGroup(64, 36), 15 );;
gap> Print(List([1..14], i->Homology(TensorWithIntegersModP(R, 2), i)), "\n");
[ 2, 3, 5, 6, 7, 9, 11, 14, 17, 20, 24, 27, 30, 34 ]
gap> ##
gap> ## Test ModPCohomologyRingPresentation
gap> ModPCohomologyRingPresentation(SmallGroup(64, 5), 6);
Graded algebra GF(2)[ x_1, x_2, x_3, x_4, x_5, x_6, x_7, x_8 ] /
[ x_1*x_2, x_1^2, x_1*x_4+x_2*x_3+x_2*x_5, x_1*x_3+x_1*x_5+x_2*x_3+x_2*x_5,
  x_2^3+x_2*x_3+x_2*x_5, x_2^2*x_4+x_3^2+x_5^2, x_2*x_6+x_3^2+x_5^2, x_1*x_6,
  x_2^2*x_3+x_2^2*x_5, x_1*x_7, x_2*x_3*x_4+x_2*x_4*x_5,
  x_2^2*x_7+x_2*x_3*x_5+x_2*x_5^2+x_3*x_6+x_5*x_6, x_2*x_3^2+x_2*x_5^2,
  x_2*x_4^2+x_3*x_7+x_4*x_6+x_5*x_7, x_6^2, x_2*x_3*x_7+x_2*x_5*x_7,
  x_3^3+x_3^2*x_5+x_3*x_5^2+x_5^3, x_2*x_4*x_7+x_3*x_4^2+x_4^2*x_5+x_6*x_7,
  x_2^2*x_8+x_3^2*x_5+x_3*x_4^2+x_4^3+x_4^2*x_5+x_5^3+x_6*x_7+x_7^2
 ] with indeterminate degrees [ 1, 1, 2, 2, 2, 3, 3, 4 ]

gap> ##
gap> ## Test ModPCohomologyRingPresentation
gap> ## This needs to check first of all that Singular is available
gap> ## and the HAP has been fixed
gap> ## First remember the expected answer
gap> R := PolynomialRing(GF(2), 5);;
gap> A := [ R, [ R.1^2+R.1*R.2+R.2^2, R.2^3 ], [ 1, 1, 1, 1, 4 ] ];;
gap> ## What we do now is do the test if we can, otherwise we fool ReadTest into
gap> ## printing a useful error message
gap> if not LoadPackage("Singular") = true then
>   Print("(WW) The package 'Singular' is not available: unable to\n");
>   Print("(WW) test HAPprime:ModPCohomologyRingPresentation\n");
>   res := A;
> elif not IsBound(ModPCohomologyRing(SmallGroup(8, 3), 5)!.pairToIntModified) then
>   Print("(WW) Your version of HAP does not support the extensions needed to\n");
>   Print("(WW) use HAPprime:ModPCohomologyRingPresentation. Please\n");
>   Print("(WW) upgrade to a newer version of HAP if you wish to use this function.\n");
>   res := A;
> else
>   res := ModPCohomologyRingPresentation(SmallGroup(32, 47));
> fi;
gap> Print(res, "\n");
Graded algebra PolynomialRing( GF(2), ["x_1", "x_2", "x_3", "x_4", "x_5"] ) /
[ x_3^2+x_3*x_4+x_4^2, x_4^3 ] with indeterminate degrees [ 1, 1, 1, 1, 4 ]
gap> ##
gap> ##
gap> ## Test PoincareSeriesLHS
gap> ## This needs to check first of all that Singular is available
gap> ## and the HAP has been fixed
gap> ## First remember the expected answer
gap> A := "(x_1^2+x_1+1)/(-x_1^5+x_1^4+2*x_1^3-2*x_1^2-x_1+1)";;
gap> ## What we do now is do the test if we can, otherwise we fool ReadTest into
gap> ## printing a useful error message
gap> if not LoadPackage("Singular") = true then
> Print("(WW) The package 'Singular' is not available: unable to\n");
> Print("(WW) test HAPprime:PoincareSeriesLHS\n");
> string := A;
> elif not IsBound(ModPCohomologyRing(SmallGroup(8, 3), 5)!.pairToIntModified) then
> Print("(WW) Your version of HAP does not support the extensions needed to\n");
> Print("(WW) use HAPprime:PoincareSeriesLHS. Please\n");
> Print("(WW) upgrade to a newer version of HAP if you wish to use this function.\n");
> string := A;
> else
> string := String(PoincareSeriesLHS(SmallGroup(64, 17)));
> fi;
gap> Print(string, "\n");
(x_1^2+x_1+1)/(-x_1^5+x_1^4+2*x_1^3-2*x_1^2-x_1+1)
gap> ##
gap> ##
gap> ## Now check that various problems with HAP have been fixed
gap> ##
gap> G := DihedralGroup(4);;
gap> G := G/NormalSubgroups(G)[2];;
gap> R := ResolutionPrimePowerGroup(G, 1);;
gap> if R!.dimension(1) = 2 then 
> Print("(WW) Your copy of HAP:ResolutionPrimePowerGroup gives the incorrect answer\n");
> Print("(WW) when group generators are repeated. Please upgrade to a fixed version\n");
> Print("(WW) of HAP.\n");
> fi;
gap> ##
gap> # Reset the info level
gap> SetInfoLevel(InfoHAPprime, infolevel);;
gap> 
gap> ## Tweak this number until this gives the same GAP4stones as
gap> ## ReadTest("/usr/local/lib/gap4r4/tst/combinat.tst");
gap> STOP_TEST("HAPprime/tst/happrime.tst", 4600000000);
