##############################################################################
##
#W  gp2map.gd                   GAP4 package `XMod'              Chris Wensley
#W                                                                 & Murat Alp
##
##  Declaration file for functions of the XMOD package.
##  This file contains implementations of 2dMappings for crossed modules and 
##  cat1-groups.
## 
#H  @(#)Id: "xmod/gap/gp2map.gd", v.2.11 2008/05/02 
##
#Y  Copyright (C) 2008, Chris Wensley and Murat Alp,  
#Y  School of Computer Science, Bangor University, U.K. 
#Y
#Y  This program is free software; you can redistribute it and/or
#Y  modify it under the terms of the GNU General Public License
#Y  as published by the Free Software Foundation; either version 2
#Y  of the License, or (at your option) any later version.
#Y
#Y  This program is distributed in the hope that it will be useful,
#Y  but WITHOUT ANY WARRANTY; without even the implied warranty of
#Y  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
#Y  See the GNU General Public License for more details.
#Y
#Y  You should have received a copy of the GNU General Public License along 
#Y  with this program; if not, write to the Free Software Foundation, Inc., 
#Y  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 

#############################################################################
##
#C  IsGeneral2dMapping( <map> )
#P  Is2dMapping( <mor> )
#P  IsPreXModMorphism( <mor> )
#P  IsXModMorphism( <mor> )
##
##  A pre-crossed module or pre-cat1-group morphism is a pair of group homs
##
##  (24/02/06) changed from Property to Category - is this correct??
##
DeclareCategory( "IsGeneral2dMapping", IsGeneralMapping );
DeclareCategory( "IsSPGeneral2dMapping", 
    IsGeneral2dMapping and IsSPGeneralMapping );
DeclareCategory( "IsNonSPGeneral2dMapping", 
    IsGeneral2dMapping and IsNonSPGeneralMapping );
DeclareProperty( "Is2dMapping",
    IsGeneral2dMapping and RespectsMultiplication and RespectsOne );
DeclareProperty( "IsPreXModMorphism", Is2dMapping );
DeclareProperty( "IsXModMorphism", Is2dMapping );

#############################################################################
##
#R  Is2dMappingRep( <mor> )
#A  SourceHom( <mor> )
#A  RangeHom( <mor> )
#O  Make2dMapping( <src>, <rng>, <srchom>, <rnghom> )
##
##  A pre-crossed module or pre-cat1-group morphism is a pair of group homs
##
DeclareRepresentation( "Is2dMappingRep", 
    Is2dMapping and IsAttributeStoringRep,
    [ "Source", "Range", "SourceHom", "RangeHom" ] );
DeclareAttribute( "SourceHom", Is2dMapping );
DeclareAttribute( "RangeHom", Is2dMapping );
DeclareOperation( "Make2dMapping",
    [ Is2dDomain, Is2dDomain, IsGroupHomomorphism, IsGroupHomomorphism ] );

#############################################################################
##
#A  Kernel2dMapping( <mor> )
##
DeclareAttribute( "Kernel2dMapping", Is2dMapping );

#############################################################################
##
#F  PreXModMorphism( <args> )
#O  PreXModMorphismByHoms( <src>, <rng>, <srchom>, <rnghom> )
#F  PreCat1Morphism( <args> )
#O  PreCat1MorphismByHoms( <src>, <rng>, <srchom>, <rnghom> )
##
DeclareGlobalFunction( "PreXModMorphism" );
DeclareOperation( "PreXModMorphismByHoms",
    [ IsPreXMod, IsPreXMod, IsGroupHomomorphism, IsGroupHomomorphism ] );
DeclareGlobalFunction( "PreCat1Morphism" );
DeclareOperation( "PreCat1MorphismByHoms",
    [ IsPreCat1, IsPreCat1, IsGroupHomomorphism, IsGroupHomomorphism ] );

#############################################################################
##
#F  XModMorphism( <args> )
#O  XModMorphismByHoms( <src>, <rng>, <srchom>, <rnghom> )
#F  Cat1Morphism( <args> )
#O  Cat1MorphismByHoms( <src>, <rng>, <srchom>, <rnghom> )
#O  InclusionMorphism2dDomains( <obj>, <sub> )
##
DeclareGlobalFunction( "XModMorphism" );
DeclareOperation( "XModMorphismByHoms",
    [ IsXMod, IsXMod, IsGroupHomomorphism, IsGroupHomomorphism ] );
DeclareGlobalFunction( "Cat1Morphism" );
DeclareOperation( "Cat1MorphismByHoms",
    [ IsCat1, IsCat1, IsGroupHomomorphism, IsGroupHomomorphism ] );
DeclareOperation( "InclusionMorphism2dDomains", [ Is2dDomain, Is2dDomain ] );

#############################################################################
##
#O  InnerAutomorphismXMod( <XM>, <r> )
#O  InnerAutomorphismCat1( <C1G>, <r> )
##
DeclareOperation( "InnerAutomorphismXMod",
    [ IsPreXMod, IsMultiplicativeElementWithInverse ] );
DeclareOperation( "InnerAutomorphismCat1",
    [ IsPreCat1, IsMultiplicativeElementWithInverse ] );

#############################################################################
##
#P  IsEndomorphism2dDomain( <mor> )
#P  IsAutomorphism2dDomain( <mor> )
##
DeclareProperty( "IsEndomorphism2dDomain", Is2dMapping );
DeclareProperty( "IsAutomorphism2dDomain", Is2dMapping );

#############################################################################
##
#P  IsPermPreXModMorphism( <mor> )
##
DeclareProperty( "IsPermPreXModMorphism", IsPreXModMorphism );

#############################################################################
##
#P  IsPreCat1Morphism( <mor> )
#P  IsCat1Morphism( <mor> )
#P  IsPermPreCat1Morphism( <mor> )
##
DeclareProperty( "IsPreCat1Morphism", Is2dMapping );
DeclareProperty( "IsCat1Morphism", Is2dMapping );
DeclareProperty( "IsPermPreCat1Morphism", IsPreCat1Morphism );

#############################################################################
##
#A  ReverseIsomorphism( <PCG> )
#O  CompositionMorphism( <mor2>, <mor1> )
##
DeclareAttribute( "ReverseIsomorphism", IsPreCat1 );
DeclareOperation( "CompositionMorphism", [ Is2dMapping, Is2dMapping ] );

#############################################################################
##
#P  IsSourceMorphism( <mor> )
#O  PreXModBySourceHom( <mor> )
##
DeclareProperty( "IsSourceMorphism", IsXModMorphism );
DeclareOperation( "PreXModBySourceHom", [ IsPreXModMorphism ] );

#############################################################################
##
#A  XModMorphismOfCat1Morphism( <mor> )
#O  XModMorphismByCat1Morphism( <mor> )
#A  Cat1MorphismOfXModMorphism( <mor> )
#O  Cat1MorphismByXModMorphism( <mor> )
##
DeclareAttribute( "XModMorphismOfCat1Morphism", IsCat1Morphism );
DeclareOperation( "XModMorphismByCat1Morphism", [ IsCat1Morphism ] );
DeclareAttribute( "Cat1MorphismOfXModMorphism", IsXModMorphism );
DeclareOperation( "Cat1MorphismByXModMorphism", [ IsXModMorphism ] );

#############################################################################
##
#A  IsomorphismPermPreXMod( <pre-xmod> )
#A  IsomorphismFpPreXMod( <pre-xmod> )
#A  IsomorphismPcPreXMod( <pre-xmod> )
##
DeclareAttribute( "IsomorphismPermPreXMod", IsPreXMod );
DeclareAttribute( "IsomorphismFpPreXMod", IsPreXMod );
DeclareAttribute( "IsomorphismPcPreXMod", IsPreXMod );

#############################################################################
##
#A  IsomorphismPermPreCat1( <pre-cat1> )
#A  IsomorphismFpPreCat1( <pre-cat1> )
#A  IsomorphismPcPreCat1( <pre-cat1> )
##
DeclareAttribute( "IsomorphismPermPreCat1", IsPreCat1 );
DeclareAttribute( "IsomorphismFpPreCat1", IsPreCat1 );
DeclareAttribute( "IsomorphismPcPreCat1", IsPreCat1 );

#############################################################################
##
#A  IsomorphismXModByNormalSubgroup( <xmod> )
##
DeclareAttribute( "IsomorphismXModByNormalSubgroup", IsXMod );

#############################################################################
##
#O  PreXModIsomorphismByIsomorphisms( <pre-xmod>, <sigma>, <rho> )
#F  SmallerDegreePerm2dDomain( <args> )
#A  SmallerDegreePermPreXMod( <XM> )
##
DeclareOperation( "PreXModIsomorphismByIsomorphisms", 
    [ IsPreXMod, IsBijective, IsBijective ] );
DeclareGlobalFunction( "SmallerDegreePerm2dDomain" );
DeclareAttribute( "SmallerDegreePermPreXMod", IsPreXMod );

#############################################################################
##
#E  gp2map.gd . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
