##############################################################################
##
#W  gp2obj.gi                  GAP4 package `XMod'               Chris Wensley
##
#H  @(#)$Id: "xmod/gap/gpd2obj.gi", v 2.12 2008/11/21 
##
##  Implementation file for groupoid functions in the XMOD package.
##
Revision.("xmod/gap/gpd2obj_gi") :=
              "@(#)$Id: gpd2obj.gi, v 2.12 2008/11/21 gap Exp $";

#############################################################################
##
#M  Is2dDomainWithObjects 
##
InstallMethod( Is2dDomainWithObjects, 
    "method for 2-dimensional groupoid objects", true, [ Is2dDomain ], 0,
function( obj )
    return ( HasObjectList( obj ) and HasSource( obj ) and HasRange( obj ) and
        ( IsPreXModWithObjectsObj(obj) or IsPreCat1WithObjectsObj(obj) ) );
end );

##############################################################################
##
#M  PreXModWithObjectsObj( <obs>, <bdy>, <act> ) . . . make pre-crossed module
##
InstallMethod( PreXModWithObjectsObj, "for objects, morphism and action", true,
    [ IsHomogeneousList, IsGroupoidHomomorphism, IsGroupoidHomomorphism ], 0,
function( obs, bdy, act )

    local  filter, fam, PM, ok, src, rng, aut, name;

    fam := FamilyObj( [ obs, bdy, act ] );
    filter := IsPreXModWithObjectsObj; 
    if not IsConstantOnObjects( bdy ) then 
        Error( "objects not fixed by the boundary" ); 
    fi;
    src := Source( bdy );
    rng := Range( bdy );
    if not ( rng = Source( act ) ) then
        Error( "require Range( bdy ) = Source( act )" );
    fi;
    aut := Range( act );
    if not IsGroupOfAutomorphisms( aut ) then
        Error( "Range( act ) must be a group of automorphisms" );
    fi;
    if ( IsPermGroupoid( src ) and IsPermGroupoid( rng ) ) then
        filter := filter and IsPermPreXMod;
    fi;
    PM := rec();
    ObjectifyWithAttributes( PM, 
      NewType( fam, filter ),
      ObjectList, obs, 
      Source, src,
      Range, rng,
      Boundary, bdy,
      AutoGroup, aut,
      XModAction, act,
      Is2dDomain, true );
    if not IsPreXMod( PM ) then
        Info( InfoXMod, 1, "Warning: not a pre-crossed module." );
    fi;
    ok := IsXMod( PM );
    # name := Name( PM );
    return PM;
end );

##############################################################################
##
#M  \=( <P>, <Q> )  . . . . . . . .  test if two pre-crossed modules are equal
##
InstallMethod( \=, "generic method for two pre-crossed modules of groupoids",
    IsIdenticalObj, [ IsPreXModWithObjectsObj, IsPreXModWithObjectsObj ], 0,
    function ( P, Q )
    return ( ( ObjectList(P) = ObjectList(Q) ) 
             and ( Boundary(P) = Boundary(Q) )
             and ( XModAction(P) = XModAction(Q) ) );
end );

#############################################################################
##
#E  gpd2obj.gi . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
